@@ -1,7 +1,7 @@
 ################################################################################
 # MIT License
 #
-# Copyright (c) 2020 Hajime Nakagami
+# Copyright (c) 2020-2021 Hajime Nakagami
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,23 +27,23 @@ import heapq
 
 class Node:
 
-    __slots__ = ["entry", "node_len", "pos", "epos", "index", "left_id", "right_id", "cost", "min_cost", "back_pos", "back_index", "skip"]
+    __slots__ = ["original", "feature", "node_len", "pos", "epos", "index", "left_id", "right_id", "cost", "min_cost", "back_pos", "back_index", "skip"]
 
     @classmethod
     def create_bos(cls):
-        return cls(None, 0, 1, 0, -1, 0, 0, 0, -1, -1, False)
+        return cls(None, None, 0, 1, 0, -1, 0, 0, 0, -1, -1, False)
 
     @classmethod
     def create_eos(cls, pos):
-        return cls(None, pos, pos + 1, 0, 0, -1, 0, 0x7FFFFFFF, -1, -1, False)
+        return cls(None, None, pos, pos + 1, 0, 0, -1, 0, 0x7FFFFFFF, -1, -1, False)
 
     @classmethod
     def create_by_entry(cls, e):
-        original, lc_attr, rc_attr, posid, wcost, feature, skip = e
-        return cls(e, 0, 0, posid, lc_attr, rc_attr, wcost, 0x7FFFFFFF, -1, -1, skip)
+        return cls(e.original, e.feature, 0, 0, e.posid, e.lc_attr, e.rc_attr, e.wcost, 0x7FFFFFFF, -1, -1, e.skip)
 
-    def __init__(self, entry, pos, epos, index, left_id, right_id, cost, min_cost, back_pos, back_index, skip):
-        self.entry = entry
+    def __init__(self, original, feature, pos, epos, index, left_id, right_id, cost, min_cost, back_pos, back_index, skip):
+        self.original = original
+        self.feature = feature
         self.pos = pos
         self.epos = epos
         self.index = index
@@ -54,13 +54,19 @@ class Node:
         self.back_pos = back_pos
         self.back_index = back_index
         self.skip = skip
-        self.node_len = len(self.entry[0]) if self.entry else 1     # 1: BOS or EOS
+        self.node_len = len(self.original) if self.original else 1     # 1: BOS or EOS
 
     def is_bos(self):
-        return self.entry is None and self.pos == 0
+        return self.original is None and self.pos == 0
 
     def is_eos(self):
-        return self.entry is None and self.pos != 0
+        return self.original is None and self.pos != 0
+
+    def __repr__(self):
+        original = self.original.decode('utf-8') if self.original else ""
+        feature = self.feature.decode('utf-8') if self.feature else ""
+        return f"{original},{feature},{self.node_len},{self.pos},{self.epos},{self.index},{self.left_id},{self.right_id},{self.cost},{self.min_cost},{self.back_pos},{self.back_index},{self.skip}"
+
 
 class Lattice:
     def __init__(self, size):
@@ -130,37 +136,59 @@ class Lattice:
         while pq and n:
             bp = heapq.heappop(pq)
             if bp.is_complete():
-                bp.path.reverse()
-                paths.append(bp.path)
+                bp.back_path.reverse()
+                paths.append(bp.back_path)
                 n -= 1
             else:
-                node = bp.path[-1]
+                node = bp.back_path[-1]
                 epos = node.epos - node.node_len
-                for index in range(len(self.enodes[epos])):
-                    node = self.enodes[epos][index]
+                for i in range(len(self.enodes[epos])):
+                    node = self.enodes[epos][i]
                     heapq.heappush(pq, BackwardPath(matrix, node, bp))
         return paths
 
+    def _dump_nodes_list(self, prompt, nodes_list):
+        print("+" * 110)
+        print(prompt)
+        for nodes in nodes_list:
+            print("-" * 110)
+            self._dump_nodes(nodes)
+
+    def _dump_nodes(self, nodes):
+        print("-" * 110)
+        for node in nodes:
+            print(node)
+
+    def dump_snodes_list(self):
+        # for debug
+        self._dump_nodes_list("snodes", self.snodes)
+
+    def dump_enodes_list(self):
+        # for debug
+        self._dump_nodes_list("enodes", self.enodes)
+
 
 class BackwardPath:
     def __init__(self, matrix, node, right_path=None):
         self.cost_from_bos = node.min_cost
-        if right_path is None:
-            assert node.is_eos()
-            self.cost_from_eos = 0
-            self.path = [node]
-        else:
-            neighbor_node = right_path.path[-1]
+        self.cost_from_eos = 0
+        self.back_path = []
+
+        if right_path is not None:
+            neighbor_node = right_path.back_path[-1]
             self.cost_from_eos = (
                 right_path.cost_from_eos +
                 neighbor_node.cost +
                 matrix.get_trans_cost(node.right_id, neighbor_node.left_id)
             )
-            self.path = right_path.path[:]
-            self.path.append(node)
+            self.back_path = right_path.back_path[:]
+        else:
+            assert node.is_eos()
+
+        self.back_path.append(node)
 
     def __lt__(self, other):
         return self.cost_from_bos + self.cost_from_eos < other.cost_from_bos + other.cost_from_eos
 
     def is_complete(self):
-        return self.path[-1].is_bos()
+        return self.back_path[-1].is_bos()
