@@ -1,7 +1,7 @@
 ################################################################################
 # MIT License
 #
-# Copyright (c) 2020 Hajime Nakagami
+# Copyright (c) 2020-2021 Hajime Nakagami
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -69,6 +69,10 @@ class Tokenizer:
             pos += lat.forward()
 
         lat.end(self.matrix)
+
+        # lat.dump_snodes_list()
+        # lat.dump_enodes_list()
+
         return lat
 
     def tokenize(self, s):
@@ -78,7 +82,7 @@ class Tokenizer:
         morphemes = []
         for node in nodes[1:-1]:
             morphemes.append(
-                (node.entry[0].decode('utf-8'), node.entry[5].decode('utf-8'))
+                (node.original.decode('utf-8'), node.feature.decode('utf-8'))
             )
         return morphemes
 
@@ -89,7 +93,7 @@ class Tokenizer:
             morphemes = []
             for node in nodes[1:-1]:
                 morphemes.append(
-                    (node.entry[0].decode('utf-8'), node.entry[5].decode('utf-8'))
+                    (node.original.decode('utf-8'), node.feature.decode('utf-8'))
                 )
             morphemes_list.append(morphemes)
 
